# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["ArtifactRef"]


class ArtifactRef(BaseModel):
    artifact_id: str

    name: str
